==> security-1.0.20-sources.jar: security/provider/AuthenticationProvider.java <==
package security.provider;

import framework.captcha.Captcha;
import framework.config.SecurityConfig;
import framework.security.Account;
import framework.security.AccountLoader;
import framework.security.RegApproval;
import framework.security.password.PasswordService;
import framework.utils.RequestUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import security.exceptions.CaptchaException;
import security.service.CaptchaFailCache;
import security.vo.UserDetail;

import javax.servlet.http.HttpServletRequest;

/**
 * 登入处理器(附加验证码，验证)
 */
@Component
public class AuthenticationProvider extends DaoAuthenticationProvider {

    @Autowired
    private Captcha captcha;
    @Autowired
    private SecurityConfig securityConfig;
    @Autowired
    private PasswordService passwordService;
    @Autowired
    private AccountLoader accountLoader;
    @Autowired
    private CaptchaFailCache captchaFailCache;

    public AuthenticationProvider(UserDetailsService userDetailsService) {
        super.setUserDetailsService(userDetailsService);
    }

    /**
     * 重载授权认证，加入验证码校验
     *
     * @param authentication
     * @return
     * @throws AuthenticationException
     */
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String _captcha = request.getParameter("captcha");
        String _captchaId = request.getParameter("captchaId");
        String username = (String) authentication.getPrincipal();

        //is captcha check
        boolean isCheckCaptcha = StringUtils.hasText(_captchaId) && StringUtils.hasText(_captcha);

        //is need captcha check
        if (!isCheckCaptcha) {
            isCheckCaptcha = this.securityConfig.getEnableLoginCaptcha() && this.captchaFailCache.hasFailed(username);
        }

        //captcha check
        if (isCheckCaptcha) {
            this.validateCaptcha(_captchaId, _captcha);
        }

        //login fail limit
        int loginFailLimit = this.accountLoader.loginFailLimit(username);
        if (loginFailLimit > 0) {
            throw new LockedException(RequestUtil.getMessageDefault("security.loginFailLimit"
                    , "Login failure too many times, limit login {0} minutes"
                    , ((int) Math.ceil(loginFailLimit / 60d)) + ""));
        }
        //invoke base
        return super.authenticate(authentication);
    }

    /**
     * 重载附加检查
     *
     * @param userDetails
     * @param authentication
     * @throws AuthenticationException
     */
    @Override
    protected void additionalAuthenticationChecks(UserDetails userDetails, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
        Account account = ((UserDetail) userDetails).getAccount();
        //password check
        if (authentication.getCredentials() == null) {
            this.logger.debug("Failed to authenticate since no credentials provided");
            throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
        } else {

            //check password
            String rawPassword = authentication.getCredentials().toString();
            String salt = account.getPasswordSalt();
            String encodePassword = userDetails.getPassword();
            if (!this.passwordService.matched(rawPassword, salt, encodePassword)) {
                this.logger.debug("Failed to authenticate since password does not match stored value");
                throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
            }

            //check registration approved
            if (account.getRegApproval() == null) {
            } else if (RegApproval.Agree.equals(account.getRegApproval())) {
            } else if (RegApproval.Reject.equals(account.getRegApproval())) {
                throw new DisabledException(RequestUtil.getMessageDefault("security.userRejectApproved", "Registration approval is rejected, the account is unavailable"));
            } else if (RegApproval.Waiting.equals(account.getRegApproval())) {
                throw new DisabledException(RequestUtil.getMessageDefault("security.userWaitingApproved", "Waiting for registration approval, the account is unavailable"));
            }
        }
    }

    /**
     * 验证码验证
     *
     * @param captchaId
     * @param captcha
     */
    protected void validateCaptcha(String captchaId, String captcha) {
        if (!StringUtils.hasText(captcha)) {
            throw new CaptchaException(RequestUtil.getMessageDefault("security.captcha.empty", "Please input captcha code"));
        }

        if (!StringUtils.hasText(captchaId)) {
            throw new CaptchaException("Not set captchaId");
        }

        boolean checkSuccess = this.captcha.check(captchaId, captcha);
        if (checkSuccess) {
            this.captcha.remove(captchaId);
        } else {
            throw new CaptchaException(RequestUtil.getMessageDefault("security.captcha.invalid", "Captcha code error"));
        }
    }
}

==> security-1.0.25-sources.jar: security/provider/AuthenticationProvider.java <==
package security.provider;

import framework.captcha.Captcha;
import framework.config.SecurityConfig;
import framework.crypto.GeneralCrypto;
import framework.security.Account;
import framework.security.AccountLoader;
import framework.security.RegApproval;
import framework.security.password.PasswordService;
import framework.utils.RequestUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import security.exceptions.CaptchaException;
import security.service.CaptchaFailCache;
import security.vo.UserDetail;

import javax.servlet.http.HttpServletRequest;

/**
 * 登入处理器(附加验证码，验证)
 */
@Component
public class AuthenticationProvider extends DaoAuthenticationProvider {

    @Autowired
    private Captcha captcha;
    @Autowired
    private SecurityConfig securityConfig;
    @Autowired
    private PasswordService passwordService;
    @Autowired
    private AccountLoader accountLoader;
    @Autowired
    private CaptchaFailCache captchaFailCache;
    @Autowired
    private GeneralCrypto generalCrypto;

    public AuthenticationProvider(UserDetailsService userDetailsService) {
        super.setUserDetailsService(userDetailsService);
    }

    /**
     * 重载授权认证，加入验证码校验
     *
     * @param authentication
     * @return
     * @throws AuthenticationException
     */
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String _captcha = request.getParameter("captcha");
        String _captchaId = request.getParameter("captchaId");
        String username = (String) authentication.getPrincipal();

        //is captcha check
        boolean isCheckCaptcha = StringUtils.hasText(_captchaId) && StringUtils.hasText(_captcha);

        //is need captcha check
        if (!isCheckCaptcha) {
            isCheckCaptcha = this.securityConfig.getEnableLoginCaptcha() && this.captchaFailCache.hasFailed(username);
        }

        //captcha check
        if (isCheckCaptcha) {
            this.validateCaptcha(_captchaId, _captcha);
        }

        //login fail limit
        int loginFailLimit = this.accountLoader.loginFailLimit(username);
        if (loginFailLimit > 0) {
            throw new LockedException(RequestUtil.getMessageDefault("security.loginFailLimit"
                    , "Login failure too many times, limit login {0} minutes"
                    , ((int) Math.ceil(loginFailLimit / 60d)) + ""));
        }
        //invoke base
        return super.authenticate(authentication);
    }

    /**
     * 重载附加检查
     *
     * @param userDetails
     * @param authentication
     * @throws AuthenticationException
     */
    @Override
    protected void additionalAuthenticationChecks(UserDetails userDetails, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
        Account account = ((UserDetail) userDetails).getAccount();
        //password check
        if (authentication.getCredentials() == null) {
            this.logger.debug("Failed to authenticate since no credentials provided");
            throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
        } else {
            String rawPassword = authentication.getCredentials().toString();

            //
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String passwordCipher = request.getParameter("passwordCipher");
            if ("gc".equals(passwordCipher)) {
                try {
                    rawPassword = this.generalCrypto.decryptFromBase64AsString(rawPassword);
                } catch (Exception exception) {
                    throw new BadCredentialsException("Password encryption transmission failed, please refresh page and submit again", exception);
                }
            }

            //check password
            String salt = account.getPasswordSalt();
            String encodePassword = userDetails.getPassword();
            if (!this.passwordService.matched(rawPassword, salt, encodePassword)) {
                this.logger.debug("Failed to authenticate since password does not match stored value");
                throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
            }

            //check registration approved
            if (account.getRegApproval() == null) {
            } else if (RegApproval.Agree.equals(account.getRegApproval())) {
            } else if (RegApproval.Reject.equals(account.getRegApproval())) {
                throw new DisabledException(RequestUtil.getMessageDefault("security.userRejectApproved", "Registration approval is rejected, the account is unavailable"));
            } else if (RegApproval.Waiting.equals(account.getRegApproval())) {
                throw new DisabledException(RequestUtil.getMessageDefault("security.userWaitingApproved", "Waiting for registration approval, the account is unavailable"));
            }
        }
    }

    /**
     * 验证码验证
     *
     * @param captchaId
     * @param captcha
     */
    protected void validateCaptcha(String captchaId, String captcha) {
        if (!StringUtils.hasText(captcha)) {
            throw new CaptchaException(RequestUtil.getMessageDefault("security.captcha.empty", "Please input captcha code"));
        }

        if (!StringUtils.hasText(captchaId)) {
            throw new CaptchaException("Not set captchaId");
        }

        boolean checkSuccess = this.captcha.check(captchaId, captcha);
        if (checkSuccess) {
            this.captcha.remove(captchaId);
        } else {
            throw new CaptchaException(RequestUtil.getMessageDefault("security.captcha.invalid", "Captcha code error"));
        }
    }
}
